@@ -1,8 +1,8 @@
 //! ASN.1 `SET OF` support.
 
 use crate::{
-    Any, ByteSlice, Decodable, Decoder, Encodable, Encoder, Error, ErrorKind, Length, Result, Tag,
-    Tagged,
+    asn1::Any, ByteSlice, Decodable, Decoder, Encodable, Encoder, Error, ErrorKind, Length, Result,
+    Tag, Tagged,
 };
 use core::{convert::TryFrom, marker::PhantomData};
 
@@ -56,7 +56,6 @@ where
     /// Create a new [`SetOfRef`] from a slice.
     pub fn new(slice: &'a [u8]) -> Result<Self> {
         let inner = ByteSlice::new(slice).map_err(|_| ErrorKind::Length { tag: Self::TAG })?;
-
         let mut decoder = Decoder::new(slice);
         let mut last_value = None;
 
@@ -67,7 +66,7 @@ where
 
             if let Some(last) = last_value.as_ref() {
                 if last >= &value {
-                    return Err(ErrorKind::Noncanonical.into());
+                    return Err(Self::TAG.non_canonical_error());
                 }
             }
 
@@ -210,7 +209,7 @@ where
 
             if let Some(last) = last_value.take() {
                 if last >= value {
-                    return Err(ErrorKind::Noncanonical.into());
+                    return Err(Self::TAG.non_canonical_error());
                 }
 
                 result.insert(last);
