@@ -5,14 +5,15 @@
 //!
 //! This crate provides a `no_std`-friendly implementation of a subset of ASN.1
 //! DER necessary for decoding/encoding various cryptography-related formats
-//! implemented as part of the [RustCrypto] project, e.g. the [`pkcs8`] crate.
+//! implemented as part of the [RustCrypto] project, e.g. the [`pkcs5`] and
+//! [`pkcs8`] crates.
 //!
 //! The core implementation avoids any heap usage (with convenience methods
 //! that allocate gated under the off-by-default `alloc` feature).
 //!
 //! # Minimum Supported Rust Version
 //!
-//! This crate requires **Rust 1.47** at a minimum.
+//! This crate requires **Rust 1.51** at a minimum.
 //!
 //! We may change the MSRV in the future, but it will be accompanied by a minor
 //! version bump.
@@ -29,7 +30,8 @@
 //!
 //! - `()`: ASN.1 `NULL` (see also [`Null`])
 //! - [`bool`]: ASN.1 `BOOLEAN`
-//! - [`i8`], [`i16`], [`u8`], [`u16`]: ASN.1 `INTEGER`
+//! - [`i8`], [`i16`], [`i32`], [`i64`], [`i128`]: ASN.1 `INTEGER`
+//! - [`u8`], [`u16`], [`u32`], [`u64`], [`u128`]: ASN.1 `INTEGER`
 //! - [`str`], [`String`][`alloc::string::String`]: ASN.1 `UTF8String`
 //!   (see also [`Utf8String`]. `String` requires `alloc` feature)
 //! - [`BTreeSet`][`alloc::collections::BTreeSet`]: ASN.1 `SET OF` (requires `alloc` feature)
@@ -39,7 +41,6 @@
 //! The following ASN.1 types provided by this crate also impl these traits:
 //!
 //! - [`Any`]: ASN.1 `ANY`
-//! - [`BigUInt`]: ASN.1 unsigned `INTEGER` with raw access to encoded bytes
 //! - [`BitString`]: ASN.1 `BIT STRING`
 //! - [`GeneralizedTime`]: ASN.1 `GeneralizedTime`
 //! - [`Ia5String`]: ASN.1 `IA5String`
@@ -49,6 +50,7 @@
 //! - [`PrintableString`]: ASN.1 `PrintableString` (ASCII subset)
 //! - [`Sequence`]: ASN.1 `SEQUENCE`
 //! - [`SetOfRef`]: ASN.1 `SET OF`
+//! - [`UIntBytes`]: ASN.1 unsigned `INTEGER` with raw access to encoded bytes
 //! - [`UtcTime`]: ASN.1 `UTCTime`
 //! - [`Utf8String`]: ASN.1 `UTF8String`
 //!
@@ -86,7 +88,10 @@
 //! // It does leverage the `alloc` feature, but also provides instructions for
 //! // "heapless" usage when the `alloc` feature is disabled.
 //! use core::convert::{TryFrom, TryInto};
-//! use der::{Any, Decodable, Encodable, Message, ObjectIdentifier};
+//! use der::{
+//!     asn1::{Any, ObjectIdentifier},
+//!     Decodable, Encodable, Message
+//! };
 //!
 //! /// X.509 `AlgorithmIdentifier`.
 //! #[derive(Copy, Clone, Debug, Eq, PartialEq)]
@@ -102,11 +107,11 @@
 //! // Note: types which impl `TryFrom<Any<'a>, Error = der::Error>` receive a
 //! // blanket impl of the `Decodable` trait, therefore satisfying the
 //! // `Decodable` trait bounds on `Message`, which is impl'd below.
-//! impl<'a> TryFrom<der::Any<'a>> for AlgorithmIdentifier<'a> {
+//! impl<'a> TryFrom<Any<'a>> for AlgorithmIdentifier<'a> {
 //!    type Error = der::Error;
 //!
-//!     fn try_from(any: der::Any<'a>) -> der::Result<AlgorithmIdentifier> {
-//!         // The `der::Any::sequence` method asserts that a `der::Any` value
+//!     fn try_from(any: Any<'a>) -> der::Result<AlgorithmIdentifier> {
+//!         // The `Any::sequence` method asserts that an `Any` value
 //!         // contains an ASN.1 `SEQUENCE` then calls the provided `FnOnce`
 //!         // with a `der::Decoder` which can be used to decode it.
 //!         any.sequence(|decoder| {
@@ -151,7 +156,7 @@
 //!     // message serialization.
 //!     //
 //!     // Trait objects are used because they allow for slices containing
-//!     // heterogenous field types, and a callback is used to allow for the
+//!     // heterogeneous field types, and a callback is used to allow for the
 //!     // construction of temporary field encoder types. The latter means
 //!     // that the fields of your Rust struct don't necessarily need to
 //!     // impl the `Encodable` trait, but if they don't you must construct
@@ -171,8 +176,8 @@
 //! // Example parameters value: OID for the NIST P-256 elliptic curve.
 //! let parameters = "1.2.840.10045.3.1.7".parse::<ObjectIdentifier>().unwrap();
 //!
-//! // We need to convert `parameters` into a `der::Any<'a>` type, which wraps
-//! // a `&'a [u8]` byte slice.
+//! // We need to convert `parameters` into an `Any<'a>` type, which wraps a
+//! // `&'a [u8]` byte slice.
 //! //
 //! // To do that, we need owned DER-encoded data so that we can have
 //! // `Any` borrow a reference to it, so we have to serialize the OID.
@@ -190,7 +195,7 @@
 //!     // OID for `id-ecPublicKey`, if you're curious
 //!     algorithm: "1.2.840.10045.2.1".parse().unwrap(),
 //!
-//!     // `der::Any<'a>` impls `TryFrom<&'a [u8]>`, which parses the provided
+//!     // `Any<'a>` impls `TryFrom<&'a [u8]>`, which parses the provided
 //!     // slice as an ASN.1 DER-encoded message.
 //!     parameters: Some(der_encoded_parameters.as_slice().try_into().unwrap())
 //! };
@@ -200,7 +205,7 @@
 //! //
 //! // As mentioned earlier, if you don't have the `alloc` feature enabled you
 //! // can create a fix-sized array instead, then call `Encoder::new` with a
-//! // refernce to it, then encode the message using
+//! // reference to it, then encode the message using
 //! // `encoder.encode(algorithm_identifier)`, then finally `encoder.finish()`
 //! // to obtain a byte slice containing the encoded message.
 //! let der_encoded_algorithm_identifier = algorithm_identifier.to_vec().unwrap();
@@ -232,7 +237,7 @@
 //! ```
 //! # #[cfg(all(feature = "alloc", feature = "derive", feature = "oid"))]
 //! # {
-//! use der::{Any, Encodable, Decodable, Message, ObjectIdentifier};
+//! use der::{asn1::{Any, ObjectIdentifier}, Encodable, Decodable, Message};
 //! use core::convert::TryInto;
 //!
 //! /// X.509 `AlgorithmIdentifier` (same as above)
@@ -254,7 +259,7 @@
 //!     // OID for `id-ecPublicKey`, if you're curious
 //!     algorithm: "1.2.840.10045.2.1".parse().unwrap(),
 //!
-//!     // `der::Any<'a>` impls `TryFrom<&'a [u8]>`, which parses the provided
+//!     // `Any<'a>` impls `TryFrom<&'a [u8]>`, which parses the provided
 //!     // slice as an ASN.1 DER-encoded message.
 //!     parameters: Some(der_encoded_parameters.as_slice().try_into().unwrap())
 //! };
@@ -286,7 +291,7 @@
 //! ```rust
 //! # #[cfg(all(feature = "alloc", feature = "derive", feature = "oid"))]
 //! # {
-//! # use der::{Any, Message, ObjectIdentifier};
+//! # use der::{asn1::{Any, ObjectIdentifier}, Message};
 //! #
 //! # #[derive(Copy, Clone, Debug, Eq, PartialEq, Message)]
 //! # pub struct AlgorithmIdentifier<'a> {
@@ -315,18 +320,33 @@
 //!
 //! [X.690]: https://www.itu.int/rec/T-REC-X.690/
 //! [RustCrypto]: https://github.com/rustcrypto
+//! [`pkcs5`]: https://docs.rs/pkcs5/
 //! [`pkcs8`]: https://docs.rs/pkcs8/
 //! [RustCrypto/utils#370]: https://github.com/RustCrypto/utils/issues/370
 //! [RFC 5280 Section 4.1.1.2]: https://tools.ietf.org/html/rfc5280#section-4.1.1.2
 //! [A Layman's Guide to a Subset of ASN.1, BER, and DER]: https://luca.ntop.org/Teaching/Appunti/asn1.html
 //! [A Warm Welcome to ASN.1 and DER]: https://letsencrypt.org/docs/a-warm-welcome-to-asn1-and-der/
+//!
+//! [`Any`]: asn1::Any
+//! [`UIntBytes`]: asn1::UIntBytes
+//! [`BitString`]: asn1::BitString
+//! [`GeneralizedTime`]: asn1::GeneralizedTime
+//! [`Ia5String`]: asn1::Ia5String
+//! [`Null`]: asn1::Null
+//! [`ObjectIdentifier`]: asn1::ObjectIdentifier
+//! [`OctetString`]: asn1::OctetString
+//! [`PrintableString`]: asn1::PrintableString
+//! [`Sequence`]: asn1::Sequence
+//! [`SetOfRef`]: asn1::SetOfRef
+//! [`UtcTime`]: asn1::UtcTime
+//! [`Utf8String`]: asn1::Utf8String
 
 #![no_std]
 #![cfg_attr(docsrs, feature(doc_cfg))]
 #![doc(
     html_logo_url = "https://raw.githubusercontent.com/RustCrypto/meta/master/logo.svg",
     html_favicon_url = "https://raw.githubusercontent.com/RustCrypto/meta/master/logo.svg",
-    html_root_url = "https://docs.rs/der/0.3.5"
+    html_root_url = "https://docs.rs/der/0.4.0"
 )]
 #![forbid(unsafe_code, clippy::unwrap_used)]
 #![warn(missing_docs, rust_2018_idioms, unused_qualifications)]
@@ -337,10 +357,11 @@ extern crate alloc;
 #[cfg(feature = "std")]
 extern crate std;
 
+pub mod asn1;
 pub mod message;
 
-mod asn1;
 mod byte_slice;
+mod choice;
 mod datetime;
 mod decodable;
 mod decoder;
@@ -353,21 +374,7 @@ mod str_slice;
 mod tag;
 
 pub use crate::{
-    asn1::{
-        any::Any,
-        bit_string::BitString,
-        choice::Choice,
-        context_specific::ContextSpecific,
-        generalized_time::GeneralizedTime,
-        ia5_string::Ia5String,
-        null::Null,
-        octet_string::OctetString,
-        printable_string::PrintableString,
-        sequence::Sequence,
-        set_of::{SetOf, SetOfRef, SetOfRefIter},
-        utc_time::UtcTime,
-        utf8_string::Utf8String,
-    },
+    choice::Choice,
     decodable::Decodable,
     decoder::Decoder,
     encodable::Encodable,
@@ -376,19 +383,15 @@ pub use crate::{
     header::Header,
     length::Length,
     message::Message,
-    tag::{Class, Tag, Tagged},
+    tag::{Class, Tag, TagNumber, Tagged},
 };
 
-pub(crate) use crate::byte_slice::ByteSlice;
-
-#[cfg(feature = "big-uint")]
-#[cfg_attr(docsrs, doc(cfg(feature = "big-uint")))]
-pub use {crate::asn1::big_uint::BigUInt, typenum::consts};
+#[cfg(feature = "bigint")]
+#[cfg_attr(docsrs, doc(cfg(feature = "bigint")))]
+pub use crypto_bigint as bigint;
 
 #[cfg(feature = "derive")]
 #[cfg_attr(docsrs, doc(cfg(feature = "derive")))]
 pub use der_derive::{Choice, Message};
 
-#[cfg(feature = "oid")]
-#[cfg_attr(docsrs, doc(cfg(feature = "oid")))]
-pub use const_oid::ObjectIdentifier;
+pub(crate) use crate::byte_slice::ByteSlice;
